@@ -69,7 +69,8 @@ import Prelude hiding (catch)
 import Network.Gitit.State
 import Text.XHtml hiding ( (</>), dir, method, password, rev )
 import qualified Text.XHtml as X ( method )
-import Data.List (intersperse, nub, sortBy, find, isPrefixOf, inits, sort)
+import Data.List (intercalate, intersperse, delete, nub, sortBy, find, isPrefixOf, inits, sort, (\\))
+import Data.List.Split (wordsBy)
 import Data.Maybe (fromMaybe, mapMaybe, isJust, catMaybes)
 import Data.Ord (comparing)
 import Data.Char (toLower, isSpace)
@@ -83,13 +84,14 @@ import Data.FileStore
 import System.Log.Logger (logM, Priority(..))
 
 handleAny :: Handler
-handleAny = uriRest $ \uri ->
+handleAny = withData $ \(params :: Params) -> uriRest $ \uri ->
   let path' = uriPath uri
   in  do fs <- getFileStore
+         let rev = pRevision params
          mimetype <- getMimeTypeForExtension
                       (takeExtension path')
          res <- liftIO $ try
-                (retrieve fs path' Nothing :: IO B.ByteString)
+                (retrieve fs path' rev :: IO B.ByteString)
          case res of
                 Right contents -> ignoreFilters >>  -- don't compress
                                   (ok $ setContentType mimetype $
@@ -141,10 +143,18 @@ createPage = do
                                     , pgTabs = []
                                     , pgTitle = "Create " ++ page ++ "?"
                                     } $
-                    p << [ stringToHtml ("There is no page '" ++ page ++
-                              "'.  You may create the page by "),
-                            anchor ! [href $ base' ++ "/_edit" ++ urlForPage page] <<
-                              "clicking here." ]
+                    (p << stringToHtml
+                        ("There is no page named '" ++ page ++ "'. You can:"))
+                        +++
+                    (unordList $
+                      [ anchor !
+                            [href $ base' ++ "/_edit" ++ urlForPage page] <<
+                              ("Create the page '" ++ page ++ "'")
+                      , anchor !
+                            [href $ base' ++ "/_search?" ++
+                                (urlEncodeVars [("patterns", page)])] <<
+                              ("Search for pages containing the text '" ++
+                                page ++ "'")])
 
 uploadForm :: Handler
 uploadForm = withData $ \(params :: Params) -> do
@@ -153,18 +163,18 @@ uploadForm = withData $ \(params :: Params) -> do
   let logMsg = pLogMsg params
   let upForm = form ! [X.method "post", enctype "multipart/form-data"] <<
        fieldset <<
-       [ p << [label << "File to upload:"
+       [ p << [label ! [thefor "file"] << "File to upload:"
               , br
               , afile "file" ! [value origPath] ]
-       , p << [ label << "Name on wiki, including extension"
+       , p << [ label ! [thefor "wikiname"] << "Name on wiki, including extension"
               , noscript << " (leave blank to use the same filename)"
               , stringToHtml ":"
               , br
               , textfield "wikiname" ! [value wikiname]
               , primHtmlChar "nbsp"
               , checkbox "overwrite" "yes"
-              , label << "Overwrite existing file" ]
-       , p << [ label << "Description of content or changes:"
+              , label ! [thefor "overwrite"] << "Overwrite existing file" ]
+       , p << [ label ! [thefor "logMsg"] << "Description of content or changes:"
               , br
               , textfield "logMsg" ! [size "60", value logMsg]
               , submit "upload" "Upload" ]
@@ -377,10 +387,12 @@ showHistory file page params =  do
 
 showActivity :: Handler
 showActivity = withData $ \(params :: Params) -> do
+  cfg <- getConfig
   currTime <- liftIO getCurrentTime
-  let oneMonthAgo = addUTCTime (-60 * 60 * 24 * 30) currTime
+  let defaultDaysAgo = fromIntegral (recentActivityDays cfg)
+  let daysAgo = addUTCTime (defaultDaysAgo * (-60) * 60 * 24) currTime
   let since = case pSince params of
-                   Nothing -> Just oneMonthAgo
+                   Nothing -> Just daysAgo
                    Just t  -> Just t
   let forUser = pForUser params
   fs <- getFileStore
@@ -392,14 +404,13 @@ showActivity = withData $ \(params :: Params) -> do
   let fileFromChange (Added f)    = f
       fileFromChange (Modified f) = f
       fileFromChange (Deleted f)  = f
-  let dropDotPage file = if isPageFile file
-                            then dropExtension file
-                            else file
+
   base' <- getWikiBase
-  let fileAnchor revis file =
-        anchor ! [href $ base' ++ "/_diff" ++ urlForPage file ++ "?to=" ++ revis] << file
-  let filesFor changes revis = intersperse (primHtmlChar "nbsp") $
-        map (fileAnchor revis . dropDotPage . fileFromChange) changes
+  let fileAnchor revis file = if isPageFile file
+        then anchor ! [href $ base' ++ "/_diff" ++ urlForPage (dropExtension file) ++ "?to=" ++ revis] << dropExtension file
+        else anchor ! [href $ base' ++ urlForPage file ++ "?revision=" ++ revis] << file
+  let filesFor changes revis = intersperse (stringToHtml " ") $
+        map (fileAnchor revis . fileFromChange) changes
   let heading = h1 << ("Recent changes by " ++ fromMaybe "all users" forUser)
   let revToListItem rev = li <<
         [ thespan ! [theclass "date"] << (show $ revDateTime rev)
@@ -464,7 +475,8 @@ showDiff file page params = do
                                           pgMessages = pMessages params,
                                           pgTabs = DiffTab :
                                                    pgTabs defaultPageLayout,
-                                          pgSelectedTab = DiffTab
+                                          pgSelectedTab = DiffTab,
+                                          pgTitle = page
                                           }
                                        htmlDiff
 
@@ -523,7 +535,7 @@ editPage' params = do
                    , textarea ! (readonly ++ [cols "80", name "editedText",
                                   identifier "editedText"]) << raw
                    , br
-                   , label << "Description of changes:"
+                   , label ! [thefor "logMsg"] << "Description of changes:"
                    , br
                    , textfield "logMsg" ! (readonly ++ [value logMsg])
                    , submit "update" "Save"
@@ -599,7 +611,7 @@ deletePage = withData $ \(params :: Params) -> do
   if pConfirm params && (file == page || file == page <.> "page")
      then do
        fs <- getFileStore
-       liftIO $ delete fs file author descrip
+       liftIO $ Data.FileStore.delete fs file author descrip
        seeOther (base' ++ "/") $ toResponse $ p << "File deleted"
      else seeOther (base' ++ urlForPage page) $ toResponse $ p << "Not deleted"
 
@@ -674,8 +686,10 @@ fileListToHtml base' prefix files =
         li ! [theclass "page"  ] <<
           anchor ! [href $ base' ++ urlForPage (prefix ++ dropExtension f)] <<
             dropExtension f
-      fileLink (FSFile f) =
-        li ! [theclass "upload"] << anchor ! [href $ base' ++ urlForPage (prefix ++ f)] << f
+      fileLink (FSFile f) = li ! [theclass "upload"] << concatHtml
+        [ anchor ! [href $ base' ++ urlForPage (prefix ++ f)] << f
+        , anchor ! [href $ base' ++ "_delete" ++ urlForPage (prefix ++ f)] << "(delete)"
+        ]
       fileLink (FSDirectory f) =
         li ! [theclass "folder"] <<
           anchor ! [href $ base' ++ urlForPage (prefix ++ f) ++ "/"] << f
@@ -694,30 +708,44 @@ fileListToHtml base' prefix files =
 -- more sophisticated searching options to filestore.
 categoryPage :: Handler
 categoryPage = do
-  category <- getPath
+  path' <- getPath
   cfg <- getConfig
+  let pcategories = wordsBy (==',') path'
   let repoPath = repositoryPath cfg
-  let categoryDescription = "Category: " ++ category
+  let categoryDescription = "Category: " ++ (intercalate " + " pcategories)
   fs <- getFileStore
   files <- liftIO $ index fs
   let pages = filter (\f -> isPageFile f && not (isDiscussPageFile f)) files
   matches <- liftM catMaybes $
              forM pages $ \f -> do
                categories <- liftIO $ readCategories $ repoPath </> f
-               return $ if category `elem` categories
-                           then Just f
+               return $ if all ( `elem` categories) pcategories
+                           then Just (f, categories \\ pcategories)
                            else Nothing
   base' <- getWikiBase
   let toMatchListItem file = li <<
         [ anchor ! [href $ base' ++ urlForPage (dropExtension file)] << dropExtension file ]
-  let htmlMatches = ulist << map toMatchListItem matches
+  let toRemoveListItem cat = li << 
+        [ anchor ! [href $ base' ++
+        (if null (tail pcategories)
+         then "/_categories"
+         else "/_category" ++ urlForPage (intercalate "," $ Data.List.delete cat pcategories)) ]
+        << ("-" ++ cat) ]
+  let toAddListItem cat = li <<
+        [ anchor ! [href $ base' ++
+          "/_category" ++ urlForPage (path' ++ "," ++ cat) ]
+        << ("+" ++ cat) ]
+  let matchList = ulist << map toMatchListItem (fst $ unzip matches) +++
+                  thediv ! [ identifier "categoryList" ] <<
+                  ulist << (++) (map toAddListItem (nub $ concat $ snd $ unzip matches)) 
+                                (map toRemoveListItem pcategories) 
   formattedPage defaultPageLayout{
                   pgPageName = categoryDescription,
                   pgShowPageTools = False,
                   pgTabs = [],
                   pgScripts = ["search.js"],
                   pgTitle = categoryDescription }
-                htmlMatches
+                matchList
 
 categoryListPage :: Handler
 categoryListPage = do
